//////////////////////////////////////////////////////////////////////////
//
// $Id: QAvatar.cxx,v 1.7 2004/03/26 20:56:19 scharver Exp $
//
// Author: Chris Scharver
// Email: scharver@evl.uic.edu
// Copyright (c) 2003 Electronic Visualization Laboratory,
//                    University of Illinois at Chicago
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//
//////////////////////////////////////////////////////////////////////////

#include <QAvatar.h>

#include <assert.h>
#include <string.h>
#include <QUANTA/QUANTAnet_socketbase_c.hxx>
#include <QUANTA/QUANTAts_mutex_c.hxx>

#ifdef USE_STL_NAMESPACE
#include <iostream>
using std::cerr;
using std::endl;
#else /* USE_STL_NAMESPACE */
#include <iostream.h>
#endif /* USE_STL_NAMESPACE */

QAvatar::QAvatar(const long& avID)
  : m_id(avID), m_lastUpdateTime(0.0), m_mutex(new QUANTAts_mutex_c),
    m_auxSize(0), m_helloSize(0), m_trackerSize(0),
    m_auxData(NULL), m_helloData(NULL), m_trackerData(NULL)
{
}

/** The parameterized constructor initializes the instance's constants using
    its paramters. The order is important, as other the data sizes will not
    be correct during program execution. This constructor is preferred, and
    subclasses should perform call it from their own constructors. The data
    buffer sizes may only be set once, so make sure that there is
    appropriate memory allocated.

    @param avId - unique avatar id
    @param auxSize - aux data buffer size
    @param helloSize - hello data buffer size
    @param trackerSize - tracker data buffer size
*/
QAvatar::QAvatar(const long& avID, const size_t& auxSize,
         const size_t& helloSize, const size_t& trackerSize)
  : m_id(avID), m_lastUpdateTime(0.0), m_mutex(new QUANTAts_mutex_c),
    m_auxSize(auxSize), m_helloSize(helloSize), m_trackerSize(trackerSize),
    m_auxData(NULL), m_helloData(NULL), m_trackerData(NULL)
{
}

/** The destructor deletes the avatar's m_mutex. It also deletes all data
    buffers that have been allocated for use with the avatar instance.
*/
QAvatar::~QAvatar()
{
  delete m_mutex;
  delete[] m_auxData;
  delete[] m_helloData;
  delete[] m_trackerData;
}

void
QAvatar::lock()
{
  assert(m_mutex != NULL);
  m_mutex->lock();
}

void
QAvatar::unlock()
{
  assert(m_mutex != NULL);
  m_mutex->unlock();
}

long
QAvatar::getID()
{
  return m_id;
}

char*
QAvatar::getAuxData()
{
  return m_auxData;
}

size_t
QAvatar::getAuxDataSize()
{
  return m_auxSize;
}

char*
QAvatar::getHelloData()
{
  return m_helloData;
}

size_t
QAvatar::getHelloDataSize()
{
  return m_helloSize;
}

char*
QAvatar::getTrackerData()
{
  return m_trackerData;
}

size_t
QAvatar::getTrackerDataSize()
{
  return m_trackerSize;
}

/** This method accepts a pointer to a data buffer and the size of that data.
    These values are copied into the avatar's aux data.

    @param dataBuffer pointer to a buffer
    @param dataSize size in bytes of the buffer
*/
void
QAvatar::setAuxData(const char* dataBuffer, const size_t& dataSize)
{
  if (!m_auxData) {
    //m_auxSize = dataSize;
    m_auxData = new char[m_auxSize];
  } else if (m_auxSize < dataSize) {
    cerr << "AVATAR WARNING: aux data truncated" << endl;
  }
  memcpy(m_auxData, dataBuffer, m_auxSize);
}

/** This method accepts a pointer to a data buffer and the size of that data.
    These values are copied into the avatar's hello data.

    @param dataBuffer pointer to a buffer
    @param dataSize size in bytes of the buffer
*/
void
QAvatar::setHelloData(const char* dataBuffer, const size_t& dataSize)
{
  if (!m_helloData) {
    //m_helloSize = dataSize;
    m_helloData = new char[m_helloSize];
  } else if (m_helloSize < dataSize) {
    cerr << "AVATAR WARNING: hello data truncated" << endl;
  }
  memcpy(m_helloData, dataBuffer, m_helloSize);
}

/** This method accepts a pointer to a data buffer and the size of that data.
    These values are copied into the avatar's tracker data.

    @param dataBuffer pointer to a buffer
    @param dataSize size in bytes of the buffer
*/
void
QAvatar::setTrackerData(const char* dataBuffer, const size_t& dataSize)
{
  if (!m_trackerData) {
    //m_trackerSize = dataSize;
    m_trackerData = new char[m_trackerSize];
  } else if (m_trackerSize < dataSize) {
    cerr << "AVATAR WARNING: tracker data truncated" << endl;
  }
  memcpy(m_trackerData, dataBuffer, m_trackerSize);
}

double
QAvatar::getLastUpdateTime()
{
  return m_lastUpdateTime;
}

void
QAvatar::markUpdateTime()
{
  m_lastUpdateTime = QUANTAnet_socketbase_c::getTimeInSecs();
}
