//////////////////////////////////////////////////////////////////////////
//
// $Id: QAvatar.h,v 1.6 2004/04/13 19:10:09 scharver Exp $
//
// Author: Chris Scharver
// Email: scharver@evl.uic.edu
// Copyright (c) 2003 Electronic Visualization Laboratory,
//                    University of Illinois at Chicago
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//
//////////////////////////////////////////////////////////////////////////

#ifndef _QAVATAR_H
#define _QAVATAR_H

#include <stdio.h>

class QUANTAts_mutex_c;

/** @brief Base avatar class representing multiple users.
    @author Chris Scharver
    $Revision: 1.6 $
    $Date: 2004/04/13 19:10:09 $

    This class represents a user and any associated data
    pertaining to that user that must be transferred over a
    network. Each avatar has three buffers: hello data is sent to
    other users to indicate arrival; aux data is sent to announce any
    miscellaneous message; and tracker data continuously updates
    position information. Subclasses have several key
    responsibilities:

    - Establish data buffer sizes.
    - Properly manage the data contained within data buffers.
    - Manage locking semantics if multiple threads are used.

    @sa QAvatarFactory, QAvatarListener, QAvatarManager
*/
class QAvatar
{
public:

  /// Parameterized constructor takes an identification number.
  QAvatar(const long&);

  /// Parameterized constructor takes id and data sizes.
  QAvatar(const long&, const size_t&, const size_t&, const size_t&);

  /// Virtual destructor.
  virtual ~QAvatar();

  /// Locks the avatar's mutex.
  virtual void lock();

  /// Unlocks the avatar's mutex.
  virtual void unlock();

  /// Returns the avatar's identification number.
  long getID();

  char* getAuxData();
  size_t getAuxDataSize();
  char* getHelloData();
  size_t getHelloDataSize();
  char* getTrackerData();
  size_t getTrackerDataSize();

  /// Sets the avatar's aux data.
  void setAuxData(const char*, const size_t&);

  /// Sets the avatar's hello data.
  void setHelloData(const char*, const size_t&);

  /// Sets the avatar's tracker data.
  void setTrackerData(const char*, const size_t&);

  /// Returns the last time the avatar was updated.
  double getLastUpdateTime();

  /// Marks the avatar as updated.
  void markUpdateTime();

protected:

  /// Unique identification number for the avatar.
  const long m_id;

  /// Last time the avatar was updated.
  double m_lastUpdateTime;

  /// Mutual exclusion lock can serialize multi-thread access.
  QUANTAts_mutex_c* m_mutex;

  /// @name Data Buffers
  /// @{

  /// Pointer to the avatar's aux data.
  char* m_auxData;
  /// Size in bytes of the avatar's aux data.
  const size_t m_auxSize;

  /// Pointer to the avatar's hello data.
  char* m_helloData;
  /// Size in bytes of the avatar's aux data.
  const size_t m_helloSize;

  /// Pointer to the avatar's tracker data.
  char* m_trackerData;
  /// Size in bytes of the avatar's aux data.
  const size_t m_trackerSize;

  /// @}
};

#endif /* _QAVATAR_H */
