//////////////////////////////////////////////////////////////////////////
//
// $Id: QAvatarManager.h,v 1.9 2004/04/13 19:10:10 scharver Exp $
//
// Author: Chris Scharver
// Email: scharver@evl.uic.edu
// Copyright (c) 2003 Electronic Visualization Laboratory,
//                    University of Illinois at Chicago
//
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use, copy,
// modify, merge, publish, distribute, sublicense, and/or sell copies
// of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//
//////////////////////////////////////////////////////////////////////////

#ifndef _QAVATARMANAGER_H
#define _QAVATARMANAGER_H

#include <QUANTA/QUANTAmisc_hashDict.hxx>
class QUANTAnet_tcpReflectorClient_c;
class QUANTAnet_udp_c;

#include <QAvatar.h>
class QAvatarFactory;
class QAvatarListener;

#ifdef USE_STL_NAMESPACE
#include <list>
using std::list;
#else /* USE_STL_NAMESPACE */
#include <list.h>
#endif /* USE_STL_NAMESPACE */

/** @brief Manages multiple avatar connections.
    @author Chris Scharver
    $Revision: 1.9 $
    $Date: 2004/04/13 19:10:10 $

    @sa QAvatar, QAvatarFactory, QAvatarListener
*/
class QAvatarManager
{
protected:

  static const char AUX_MSG;
  static const char BYE_MSG;
  static const char HELLO_MSG;
  static const char TRACKER_MSG;

public:

  /// Parameterized constructor requires a factory.
  QAvatarManager(QAvatarFactory*);

  /// Destructor.
  virtual ~QAvatarManager();

  /// Checks for any avatars which have timed out.
  void checkForTimeouts();

  /// Connects to the avatar server.
  virtual bool connect(const char*,
                       const unsigned short&, const unsigned short&);

  /// Returns a reference to the manager's own avatar.
  QAvatar* getSelfAvatar();

  /// Attach a new listener object.
  void attach(QAvatarListener*);

  /// Detach a listener object.
  void detach(QAvatarListener*);

  /// Process any queued network events.
  virtual void process();

  /// Send own avatar's aux data.
  virtual bool sendAux();

  /// Send goodbye message.
  virtual bool sendBye();

  /// Send own avatar's hello data.
  virtual bool sendHello();

  /// Send own avatar's tracker data.
  virtual bool sendTracker();

  void setDataSizes(const size_t&, const size_t&, const size_t&);

  /// Set the delay for sending tracker messages.
  void setTrackerUpdateDelay(const double&);

protected:

  /// Handle incoming hailing data.
  virtual void handleHailingData(char*, const size_t&);

  /// Handle incoming tracker data.
  virtual void handleTrackerData(char*, const size_t&);

  /** @name Listener Notifications */
  /// @{

  /// Notify listeners that an avatar has arrived.
  void notifyArrive(QAvatar*);

  /// Notify listeners that an avatar has new aux data.
  void notifyAux(QAvatar*);

  /// Notify listeners that an avatar has left.
  void notifyBye(QAvatar*);

  /// Notify listeners that an avatar has new hello data.
  void notifyHello(QAvatar*);

  /// Notify listeners that an avatar has new tracker data.
  void notifyTracker(QAvatar*);

  /// @}

protected:

  /// Hashed dictionary of avatars and their ids.
  QUANTAmisc_hashDict<QAvatar*, long>* m_avatarDB;

  /// The avatar representing the manager.
  QAvatar* m_selfAvatar;

  /// Pointer to the factory instance used to create new avatars.
  QAvatarFactory* m_avatarFactory;

  /// TCP channel for hailing (hello and aux) data.
  QUANTAnet_tcpReflectorClient_c* m_hailingTCP;

  /// UDP channel for tracker data.
  QUANTAnet_udp_c* m_trackerUDP;

  /// Flag indicates whether networking is enabled or not.
  bool m_noNetworking;

  char* m_hailingBuffer;
  size_t m_hailingBufSize;
  char* m_trackerBuffer;
  size_t m_trackerBufSize;

  double m_lastTimeTrackerSent;
  double m_trackerDelay;

  char* m_tempBuffer;

  typedef list<QAvatarListener*> ListenerList;
  ListenerList m_listeners;
};

#endif /* _QAVATARMANAGER_H */
